"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TNSRTCMediaStreamBase_1 = require("../base/TNSRTCMediaStreamBase");
var TNSRTCVideoTrack_1 = require("./TNSRTCVideoTrack");
var TNSRTCAudioTrack_1 = require("./TNSRTCAudioTrack");
var TNSRTCMediaStreamTrack_1 = require("./TNSRTCMediaStreamTrack");
var TNSRTCMediaStream = (function (_super) {
    __extends(TNSRTCMediaStream, _super);
    function TNSRTCMediaStream(stream) {
        var _this = _super.call(this, stream) || this;
        _this._stream = stream;
        return _this;
    }
    Object.defineProperty(TNSRTCMediaStream.prototype, "id", {
        get: function () {
            return this._stream ? this._stream.getId : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCMediaStream.prototype, "videoTracks", {
        get: function () {
            var array = [];
            var nativeArray = this._stream ? this._stream.videoTracks : [];
            var size = nativeArray.count || 0.;
            for (var i = 0; i < size; i++) {
                var track = nativeArray.objectAtIndex(i);
                array.push(TNSRTCVideoTrack_1.TNSRTCVideoTrack.fromNative(track));
            }
            return array;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCMediaStream.prototype, "audioTracks", {
        get: function () {
            var array = [];
            var nativeArray = this._stream ? this._stream.audioTracks : [];
            var size = nativeArray.count || 0;
            for (var i = 0; i < size; i++) {
                var track = nativeArray.objectAtIndex(i);
                array.push(TNSRTCAudioTrack_1.TNSRTCAudioTrack.fromNative(track));
            }
            return array;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCMediaStream.prototype, "tracks", {
        get: function () {
            var array = [];
            var nativeArray = this._stream ? this._stream.tracks : [];
            var size = nativeArray.count || 0;
            for (var i = 0; i < size; i++) {
                var track = nativeArray.objectAtIndex(i);
                array.push(TNSRTCMediaStreamTrack_1.TNSRTCMediaStreamTrack.fromNative(track));
            }
            return array;
        },
        enumerable: true,
        configurable: true
    });
    TNSRTCMediaStream.prototype.addTrack = function (track) {
        if (track instanceof TNSRTCVideoTrack_1.TNSRTCVideoTrack) {
            this._stream.addTrackWithVideo(track.instance);
        }
        if (track instanceof TNSRTCAudioTrack_1.TNSRTCAudioTrack) {
            this._stream.addTrackWithAudio(track.instance);
        }
    };
    TNSRTCMediaStream.prototype.removeTrack = function (track) {
        if (track instanceof TNSRTCVideoTrack_1.TNSRTCVideoTrack) {
            this._stream.removeTrackWithVideo(track.instance);
        }
        if (track instanceof TNSRTCAudioTrack_1.TNSRTCAudioTrack) {
            this._stream.removeTrackWithAudio(track.instance);
        }
    };
    TNSRTCMediaStream.fromNative = function (stream) {
        return new TNSRTCMediaStream(stream);
    };
    Object.defineProperty(TNSRTCMediaStream.prototype, "instance", {
        get: function () {
            return this._stream;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCMediaStream.prototype, "ios", {
        get: function () {
            return this._stream ? this._stream.stream : null;
        },
        enumerable: true,
        configurable: true
    });
    return TNSRTCMediaStream;
}(TNSRTCMediaStreamBase_1.TNSRTCMediaStreamBase));
exports.TNSRTCMediaStream = TNSRTCMediaStream;
//# sourceMappingURL=TNSRTCMediaStream.js.map