"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var TNSRTCSessionDescription_1 = require("./TNSRTCSessionDescription");
var TNSRTCSdpType_1 = require("./TNSRTCSdpType");
var TNSRTCPeerConnectionState_1 = require("./TNSRTCPeerConnectionState");
var TNSRTCIceServer_1 = require("./TNSRTCIceServer");
var TNSRTCIceCandidate_1 = require("./TNSRTCIceCandidate");
var TNSRTCDataChannel_1 = require("./TNSRTCDataChannel");
var _1 = require("./");
var TNSRTCConfiguration_1 = require("./TNSRTCConfiguration");
var TNSRTCPeerConnection = (function () {
    function TNSRTCPeerConnection(config) {
        var configuration = config && config instanceof TNSRTCConfiguration_1.TNSRTCConfiguration ? config : new TNSRTCConfiguration_1.TNSRTCConfiguration();
        this.ios = FancyRTCPeerConnection.alloc().initWithConfig(configuration.instance);
    }
    Object.defineProperty(TNSRTCPeerConnection.prototype, "localDescription", {
        get: function () {
            var localDescription = this.ios.localDescription;
            if (localDescription) {
                var type = void 0;
                switch (localDescription.type) {
                    case FancyRTCSdpType.ANSWER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.ANSWER;
                        break;
                    case FancyRTCSdpType.OFFER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.OFFER;
                        break;
                    case FancyRTCSdpType.PRANSWER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.PRANSWER;
                        break;
                    case FancyRTCSdpType.ROLLBACK:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.ROLLBACK;
                        break;
                }
                return new TNSRTCSessionDescription_1.TNSRTCSessionDescription(type, localDescription.sdp);
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCPeerConnection.prototype, "remoteDescription", {
        get: function () {
            var remoteDescription = this.ios.remoteDescription;
            if (remoteDescription) {
                var type = void 0;
                switch (remoteDescription.type) {
                    case FancyRTCSdpType.ANSWER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.ANSWER;
                        break;
                    case FancyRTCSdpType.OFFER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.OFFER;
                        break;
                    case FancyRTCSdpType.PRANSWER:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.PRANSWER;
                        break;
                    case FancyRTCSdpType.ROLLBACK:
                        type = TNSRTCSdpType_1.TNSRTCSdpType.ROLLBACK;
                        break;
                }
                return new TNSRTCSessionDescription_1.TNSRTCSessionDescription(type, remoteDescription.sdp);
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(TNSRTCPeerConnection.prototype, "connectionState", {
        get: function () {
            switch (this.ios.connectionState) {
                case FancyRTCPeerConnectionState.NEW:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.NEW;
                case FancyRTCPeerConnectionState.CONNECTING:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CONNECTING;
                case FancyRTCPeerConnectionState.CONNECTED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CONNECTED;
                case FancyRTCPeerConnectionState.DISCONNECTED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.DISCONNECTED;
                case FancyRTCPeerConnectionState.CLOSED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.CLOSED;
                case FancyRTCPeerConnectionState.FAILED:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.FAILED;
                default:
                    return TNSRTCPeerConnectionState_1.TNSRTCPeerConnectionState.NEW;
            }
        },
        enumerable: true,
        configurable: true
    });
    TNSRTCPeerConnection.prototype.onConnectionStateChange = function (callback) {
        this.ios.onConnectionStateChange(function () {
            callback();
        });
    };
    TNSRTCPeerConnection.prototype.onTrack = function (callback) {
        this.ios.onTrack(function (track) {
            callback(_1.TNSRTCTrackEvent.fromNative(track));
        });
    };
    TNSRTCPeerConnection.prototype.onRemoveTrackListener = function (callback) {
        this.ios.onRemoveTrack(function () {
            callback();
        });
    };
    TNSRTCPeerConnection.prototype.onRemoveStream = function (callback) {
        this.ios.onRemoveStream(function (nativeStream) {
            callback(_1.TNSRTCMediaStream.fromNative(nativeStream));
        });
    };
    TNSRTCPeerConnection.prototype.onIceGatheringStateChange = function (callback) {
        this.ios.onIceGatheringStateChange(function () {
            callback();
        });
    };
    TNSRTCPeerConnection.prototype.onAddStream = function (callback) {
        this.ios.onAddStream(function (nativeStream) {
            callback(_1.TNSRTCMediaStream.fromNative(nativeStream));
        });
    };
    TNSRTCPeerConnection.prototype.onNegotiationNeeded = function (callback) {
        this.ios.onNegotiationNeeded(function () {
            callback();
        });
    };
    TNSRTCPeerConnection.prototype.onSignalingStateChange = function (callback) {
        this.ios.onSignalingStateChange(function () {
            callback();
        });
    };
    TNSRTCPeerConnection.prototype.onIceCandidate = function (callback) {
        this.ios.onIceCandidate(function (nativeCandidate) {
            callback(TNSRTCIceCandidate_1.TNSRTCIceCandidate.fromNative(nativeCandidate));
        });
    };
    TNSRTCPeerConnection.prototype.onDataChannel = function (callback) {
        this.ios.onDataChannel(function (event) {
            callback(_1.TNSRTCDataChannelEvent.fromNative(event));
        });
    };
    TNSRTCPeerConnection.prototype.defaultIceServers = function () {
        var nativeServers = this.ios.defaultIceServers;
        var size = nativeServers.count;
        var servers = [];
        for (var i = 0; i < size; i++) {
            var nativeServer = nativeServers.objectAtIndex(i);
            servers.push(new TNSRTCIceServer_1.TNSRTCIceServer(nativeServer.urls, nativeServer.username, nativeServer.credential));
        }
        return servers;
    };
    TNSRTCPeerConnection.prototype.addIceCandidate = function (candidate) {
        this.ios.addIceCandidateWithCandidate(FancyRTCIceCandidate.alloc().initWithSdpSdpMidSdpMLineIndex(candidate.sdp, candidate.sdpMid, candidate.sdpMLineIndex));
    };
    TNSRTCPeerConnection.prototype.addTrack = function (track, streamIds) {
        this.ios.addTrackWithTrackStreamIds(track.mediaStreamTrack, streamIds);
    };
    TNSRTCPeerConnection.prototype.close = function () {
        this.ios.close();
    };
    TNSRTCPeerConnection.prototype.createDataChannel = function (label, channelInit) {
        var init = FancyRTCDataChannelInit.new();
        if (channelInit.id != null) {
            init.channelInit.channelId = channelInit.id;
        }
        if (channelInit.maxPacketLifeTime != null) {
            init.channelInit.maxPacketLifeTime = channelInit.maxPacketLifeTime;
        }
        if (channelInit.maxRetransmits != null) {
            init.channelInit.maxRetransmits = channelInit.maxRetransmits;
        }
        if (channelInit.negotiate != null) {
            init.channelInit.isNegotiated = channelInit.negotiate;
        }
        if (channelInit.protocol != null) {
            init.channelInit.protocol = channelInit.protocol;
        }
        if (channelInit.ordered != null) {
            init.channelInit.isOrdered = channelInit.ordered;
        }
        return TNSRTCDataChannel_1.TNSRTCDataChannel.fromNative(this.ios.createDataChannelWithLabelChannelInit(label, init));
    };
    TNSRTCPeerConnection.prototype.createAnswer = function (mediaConstraints) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var constraints = FancyRTCMediaConstraints.new();
            if (mediaConstraints.mandatory) {
                for (var _i = 0, _a = mediaConstraints.mandatory; _i < _a.length; _i++) {
                    var constraint = _a[_i];
                    var keyValue = FancyRTCKeyValue.alloc().initWithKeyValue(constraint.key, constraint.value);
                    constraints.mandatory.arrayByAddingObject(keyValue);
                }
            }
            if (mediaConstraints.optional) {
                for (var _b = 0, _c = mediaConstraints.optional; _b < _c.length; _b++) {
                    var constraint = _c[_b];
                    var keyValue = FancyRTCKeyValue.alloc().initWithKeyValue(constraint.key, constraint.value);
                    constraints.mandatory.arrayByAddingObject(keyValue);
                }
            }
            _this.ios.createAnswerWithMediaConstraintsListener(constraints, function (sdp, error) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(TNSRTCSessionDescription_1.TNSRTCSessionDescription.fromNative(sdp));
                }
            });
        });
    };
    TNSRTCPeerConnection.prototype.createOffer = function (mediaConstraints) {
        var _this = this;
        var constraints = FancyRTCMediaConstraints.new();
        if (mediaConstraints.mandatory) {
            for (var _i = 0, _a = mediaConstraints.mandatory; _i < _a.length; _i++) {
                var constraint = _a[_i];
                var keyValue = FancyRTCKeyValue.alloc().initWithKeyValue(constraint.key, constraint.value);
                constraints.mandatory.arrayByAddingObject(keyValue);
            }
        }
        if (mediaConstraints.optional) {
            for (var _b = 0, _c = mediaConstraints.optional; _b < _c.length; _b++) {
                var constraint = _c[_b];
                var keyValue = FancyRTCKeyValue.alloc().initWithKeyValue(constraint.key, constraint.value);
                constraints.mandatory.arrayByAddingObject(keyValue);
            }
        }
        return new Promise(function (resolve, reject) {
            _this.ios.createOfferWithMediaConstraintsListener(constraints, function (sdp, error) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(TNSRTCSessionDescription_1.TNSRTCSessionDescription.fromNative(sdp));
                }
            });
        });
    };
    TNSRTCPeerConnection.prototype.setLocalDescription = function (sdp) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.ios.localDescriptionWithSdpListener(sdp.instance, function (error) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve();
                }
            });
        });
    };
    TNSRTCPeerConnection.prototype.setRemoteDescription = function (sdp) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            _this.ios.remoteDescriptionWithSdpListener(sdp.instance, function (error) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve();
                }
            });
        });
    };
    TNSRTCPeerConnection.prototype.dispose = function () {
        this.ios.dispose();
    };
    return TNSRTCPeerConnection;
}());
exports.TNSRTCPeerConnection = TNSRTCPeerConnection;
//# sourceMappingURL=TNSRTCPeerConnection.js.map