"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var webrtc_common_1 = require("./webrtc.common");
var view_1 = require("tns-core-modules/ui/core/view");
var observable_1 = require("tns-core-modules/data/observable");
require("./async-await");
var ios_1 = require("./src/ios");
var webrtc_common_2 = require("./webrtc.common");
exports.IceConnectionState = webrtc_common_2.IceConnectionState;
exports.IceServer = webrtc_common_2.IceServer;
exports.IceGatheringState = webrtc_common_2.IceGatheringState;
exports.Quality = webrtc_common_2.Quality;
exports.SignalingState = webrtc_common_2.SignalingState;
exports.WebRTCDataChannelMessageType = webrtc_common_2.WebRTCDataChannelMessageType;
exports.WebRTCDataChannelState = webrtc_common_2.WebRTCDataChannelState;
exports.WebRTCSdpType = webrtc_common_2.WebRTCSdpType;
exports.WebRTCState = webrtc_common_2.WebRTCState;
__export(require("./src/ios"));
var ErrorDomain;
(function (ErrorDomain) {
    ErrorDomain["videoPermissionDenied"] = "Video permission denied";
    ErrorDomain["audioPermissionDenied"] = "Audio permission denied";
})(ErrorDomain || (ErrorDomain = {}));
var WebRTC = (function (_super) {
    __extends(WebRTC, _super);
    function WebRTC(options) {
        if (options === void 0) { options = { enableAudio: true, enableVideo: true }; }
        var _this = _super.call(this) || this;
        _this.tracks = new Map();
        _this.localStreams = new Map();
        _this.remoteStreams = new Map();
        _this._dataChannels = new Map();
        _this.remoteTracks = [];
        _this.remoteIceCandidates = [];
        _this.defaultConnectionConstraints = RTCMediaConstraints.alloc().initWithMandatoryConstraintsOptionalConstraints(null, {
            DtlsSrtpKeyAgreement: 'true',
            internalSctpDataChannels: 'true'
        });
        _this._delegate = WebRTCClientDelegate.initWithOwner(new WeakRef(_this));
        _this.configuration = RTCConfiguration.alloc().init();
        _this.configuration.bundlePolicy = 1;
        _this.configuration.continualGatheringPolicy =
            RTCContinualGatheringPolicy.GatherContinually;
        _this.configuration.tcpCandidatePolicy = RTCTcpCandidatePolicy.Disabled;
        _this.configuration.rtcpMuxPolicy = RTCRtcpMuxPolicy.Require;
        _this.configuration.keyType = RTCEncryptionKeyType.ECDSA;
        var decoder = RTCDefaultVideoDecoderFactory.alloc().init();
        var encoder = RTCDefaultVideoEncoderFactory.alloc().init();
        _this.connectionFactory = RTCPeerConnectionFactory.alloc().initWithEncoderFactoryDecoderFactory(encoder, decoder);
        var nativeIceServers = NSArray.alloc().initWithArray([]);
        if (!options.iceServers) {
            _this.defaultServers.forEach(function (server, index) {
                nativeIceServers[index] = RTCIceServer.alloc().initWithURLStrings([server]);
            });
        }
        else {
            options.iceServers.forEach(function (config, index) {
                nativeIceServers[index] = RTCIceServer.alloc().initWithURLStringsUsernameCredential([config.url], config.username, config.password);
            });
        }
        _this.configuration.iceServers = nativeIceServers;
        _this.constraints = RTCMediaConstraints.alloc().initWithMandatoryConstraintsOptionalConstraints({
            OfferToReceiveAudio: String(!!options.enableAudio),
            OfferToReceiveVideo: String(!!options.enableVideo)
        }, null);
        return _this;
    }
    Object.defineProperty(WebRTC.prototype, "delegate", {
        get: function () {
            return this._delegate;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebRTC.prototype, "dataChannels", {
        get: function () {
            return this._dataChannels;
        },
        enumerable: true,
        configurable: true
    });
    WebRTC.requestCameraPermission = function () {
        return new Promise(function (resolve, reject) {
            AVCaptureDevice.requestAccessForMediaTypeCompletionHandler(AVMediaTypeVideo, function (granted) {
                if (granted) {
                    resolve();
                }
                else {
                    reject(ErrorDomain.videoPermissionDenied);
                }
            });
        });
    };
    WebRTC.requestAudioPermission = function () {
        return new Promise(function (resolve, reject) {
            AVCaptureDevice.requestAccessForMediaTypeCompletionHandler(AVMediaTypeAudio, function (granted) {
                if (granted) {
                    resolve();
                }
                else {
                    reject(ErrorDomain.audioPermissionDenied);
                }
            });
        });
    };
    WebRTC.requestPermissions = function (explanation) {
        var _this = this;
        return new Promise(function (resolve, reject) { return __awaiter(_this, void 0, void 0, function () {
            var e_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 3, , 4]);
                        return [4, WebRTC.requestCameraPermission()];
                    case 1:
                        _a.sent();
                        return [4, WebRTC.requestAudioPermission()];
                    case 2:
                        _a.sent();
                        resolve();
                        return [3, 4];
                    case 3:
                        e_1 = _a.sent();
                        reject(e_1);
                        return [3, 4];
                    case 4: return [2];
                }
            });
        }); });
    };
    WebRTC.hasCameraPermission = function () {
        var status = AVCaptureDevice.authorizationStatusForMediaType(AVMediaTypeVideo);
        switch (status) {
            case 3:
                return true;
            default:
                return false;
        }
    };
    WebRTC.hasAudioPermission = function () {
        var status = AVCaptureDevice.authorizationStatusForMediaType(AVMediaTypeAudio);
        switch (status) {
            case 3:
                return true;
            default:
                return false;
        }
    };
    WebRTC.hasPermissions = function () {
        return this.hasCameraPermission() && this.hasAudioPermission();
    };
    WebRTC.prototype.makeOffer = function () {
        var _this = this;
        if (!this.connection)
            return;
        this.connection.offerForConstraintsCompletionHandler(this.constraints, function (sdp, error) {
            if (error) {
                _this._delegate.webRTCClientDidReceiveError(_this, error);
            }
            else {
                _this.handleSdpGenerated(sdp);
            }
        });
    };
    WebRTC.prototype.handleAnswerReceived = function (answer) {
        var _this = this;
        if (!this.connection || !answer)
            return;
        var sessionDescription = RTCSessionDescription.alloc().initWithTypeSdp(RTCSdpType.Answer, answer.sdp);
        this.connection.setRemoteDescriptionCompletionHandler(sessionDescription, function (error) {
            if (error) {
                _this._delegate.webRTCClientDidReceiveError(_this, error);
            }
            else {
                _this.handleRemoteDescriptionSet();
            }
        });
    };
    WebRTC.prototype.addIceCandidate = function (iceCandidate) {
        var nativeIceCandidate = RTCIceCandidate.alloc().initWithSdpSdpMLineIndexSdpMid(iceCandidate.sdp, iceCandidate.sdpMLineIndex, iceCandidate.sdpMid);
        if (this.connection && this.connection.remoteDescription) {
            this.connection.addIceCandidate(nativeIceCandidate);
        }
        else {
            this.remoteIceCandidates.push(nativeIceCandidate);
        }
    };
    WebRTC.prototype.createAnswerForOfferReceived = function (sdp) {
        var _this = this;
        if (!this.connection || !sdp)
            return;
        var sessionDescription = RTCSessionDescription.alloc().initWithTypeSdp(RTCSdpType.Offer, sdp.sdp);
        this.connection.setRemoteDescriptionCompletionHandler(sessionDescription, function (error) {
            if (error) {
                _this._delegate.webRTCClientDidReceiveError(_this, error);
            }
            else {
                _this.handleRemoteDescriptionSet();
                _this.connection.answerForConstraintsCompletionHandler(_this.constraints, function (generatedSdp, error) {
                    if (error) {
                        _this._delegate.webRTCClientDidReceiveError(_this, error);
                    }
                    else {
                        _this.handleSdpGenerated(generatedSdp);
                        _this._delegate.webRTCClientStartCallWithSdp(_this, generatedSdp);
                    }
                });
            }
        });
    };
    WebRTC.prototype.handleRemoteDescriptionSet = function () {
        var _this = this;
        this.remoteIceCandidates.forEach(function (iceCandidate) {
            _this.connection.addIceCandidate(iceCandidate);
        });
        this.remoteIceCandidates = [];
    };
    WebRTC.prototype.handleSdpGenerated = function (sdp) {
        var _this = this;
        if (!sdp)
            return;
        this.connection.setLocalDescriptionCompletionHandler(sdp, function (error) {
            if (error) {
                _this._delegate.webRTCClientDidReceiveError(_this, error);
            }
            else {
                _this._delegate.webRTCClientStartCallWithSdp(_this, sdp);
            }
        });
    };
    WebRTC.init = function () {
    };
    WebRTC.prototype.connect = function () {
        if (this.connection)
            return;
        this.connection = this.connectionFactory.peerConnectionWithConfigurationConstraintsDelegate(this.configuration, this.defaultConnectionConstraints, RTCPeerConnectionDelegateImpl.initWithOwner(new WeakRef(this)));
    };
    WebRTC.prototype.disconnect = function () {
        if (!this.connection)
            return;
        this.connection.close();
    };
    WebRTC.prototype.addLocalStream = function (stream) {
        if (!this.connection)
            return;
        this.connection.addStream(stream);
        this.localStreams.set(stream.streamId, stream);
    };
    WebRTC.prototype.addRemoteStream = function (stream) {
        this.remoteStreams.set(stream.streamId, stream);
    };
    WebRTC.prototype.dataChannelSend = function (name, data, type) {
        var channel = this.dataChannels.get(name);
        if (channel) {
            var isBinary = void 0;
            var nativeData = void 0;
            switch (type) {
                case webrtc_common_1.WebRTCDataChannelMessageType.TEXT:
                    isBinary = false;
                    nativeData = NSString.stringWithString(data).dataUsingEncoding(NSUTF8StringEncoding);
                    break;
                case webrtc_common_1.WebRTCDataChannelMessageType.BINARY:
                    isBinary = true;
                    nativeData = NSData.alloc().initWithBase64EncodedStringOptions(data, 0);
                    break;
            }
            var buffer = RTCDataBuffer.alloc().initWithDataIsBinary(nativeData, isBinary);
            channel.sendData(buffer);
        }
    };
    WebRTC.prototype.dataChannelClose = function (name) {
        var channel = this.dataChannels.get(name);
        if (channel) {
            channel.close();
        }
    };
    WebRTC.prototype.dataChannelCreate = function (name) {
        var config = RTCDataChannelConfiguration.alloc();
        var channel = this.connection.dataChannelForLabelConfiguration(name, config);
        this.dataChannels.set(name, channel);
        this.registerDataChannelDelegate(name);
    };
    WebRTC.prototype.switchCamera = function (trackId) {
        var mediaData = this.tracks.get(trackId);
        if (mediaData != null) {
            if (mediaData.capturer != null) {
                mediaData.capturer.toggleCamera();
            }
        }
    };
    WebRTC.prototype.registerDataChannelDelegate = function (name) {
        var channel = this.dataChannels.get(name);
        if (channel) {
            channel.delegate = RTCDataChannelDelegateImpl.initWithOwner(new WeakRef(this));
        }
    };
    WebRTC.prototype.getRandomId = function () {
        return NSUUID.UUID().UUIDString;
    };
    WebRTC.prototype.getUserMedia = function (quality) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            var factory = _this.connectionFactory;
            var streamId = _this.getRandomId();
            var localStream = factory.mediaStreamWithStreamId(streamId);
            if (!AVCaptureState.isVideoDisabled) {
                var videoSource = factory.videoSource();
                var capturer = RTCCameraVideoCapturer.alloc().initWithDelegate(videoSource);
                var customCapturer = new WebRTCCapturer(new WeakRef(_this), capturer, quality ? quality : webrtc_common_1.Quality.LOWEST);
                var videoTrackId = _this.getRandomId();
                var videoTrack = factory.videoTrackWithSourceTrackId(videoSource, videoTrackId);
                videoTrack.isEnabled = true;
                customCapturer.start();
                _this.tracks.set(videoTrackId, new MediaData(videoSource, videoTrack, customCapturer));
                localStream.addVideoTrack(videoTrack);
            }
            else {
                reject(ErrorDomain.videoPermissionDenied);
            }
            if (!AVCaptureState.isAudioDisabled) {
                var audioTrackId = _this.getRandomId();
                var audioSource = factory.audioSourceWithConstraints(RTCMediaConstraints.alloc().init());
                var audioTrack = factory.audioTrackWithSourceTrackId(audioSource, audioTrackId);
                audioTrack.isEnabled = true;
                localStream.addAudioTrack(audioTrack);
                _this.tracks.set(audioTrackId, new MediaData(audioSource, audioTrack, null));
            }
            else {
                reject(ErrorDomain.audioPermissionDenied);
            }
            resolve(localStream);
        });
    };
    WebRTC.prototype.micEnabled = function (enabled) {
    };
    WebRTC.prototype.toggleMic = function () {
    };
    WebRTC.prototype.speakerEnabled = function (enabled) {
        var audioSession = RTCAudioSession.sharedInstance;
        try {
            if (enabled) {
                audioSession.lockForConfiguration();
                audioSession.overrideOutputAudioPortError(1936747378);
                audioSession.setActiveError(true);
                audioSession.unlockForConfiguration();
            }
            else {
                audioSession.lockForConfiguration();
                audioSession.overrideOutputAudioPortError(0);
                audioSession.setActiveError(true);
                audioSession.unlockForConfiguration();
            }
        }
        catch (e) {
        }
    };
    WebRTC.prototype.enableTrack = function (trackId, enabled) {
        var mediaData = this.tracks.get(trackId);
        if (mediaData != null) {
            if (mediaData.capturer != null) {
                if (enabled) {
                    mediaData.track.isEnabled = true;
                    mediaData.capturer.start();
                }
                else {
                    mediaData.track.isEnabled = false;
                    mediaData.capturer.stop();
                }
            }
            else {
                mediaData.track.isEnabled = enabled;
            }
        }
    };
    return WebRTC;
}(webrtc_common_1.Common));
exports.WebRTC = WebRTC;
var MediaData = (function () {
    function MediaData(mediaSource, track, capturer) {
        this.mediaSource = mediaSource;
        this.track = track;
        this.capturer = capturer;
    }
    return MediaData;
}());
var AVCaptureState = (function () {
    function AVCaptureState() {
    }
    Object.defineProperty(AVCaptureState, "isVideoDisabled", {
        get: function () {
            var status = AVCaptureDevice.authorizationStatusForMediaType(AVMediaTypeVideo);
            return (status === 2 ||
                status === 1);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AVCaptureState, "isAudioDisabled", {
        get: function () {
            var status = AVCaptureDevice.authorizationStatusForMediaType(AVMediaTypeVideo);
            return (status === 2 ||
                status === 1);
        },
        enumerable: true,
        configurable: true
    });
    return AVCaptureState;
}());
var RTCDataChannelDelegateImpl = (function (_super) {
    __extends(RTCDataChannelDelegateImpl, _super);
    function RTCDataChannelDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RTCDataChannelDelegateImpl.initWithOwner = function (owner) {
        var delegate = new RTCDataChannelDelegateImpl();
        delegate._owner = owner;
        return delegate;
    };
    RTCDataChannelDelegateImpl.prototype.dataChannelDidChangeBufferedAmount = function (dataChannel, amount) {
    };
    RTCDataChannelDelegateImpl.prototype.dataChannelDidChangeState = function (dataChannel) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientDataChannelStateChanged(owner, dataChannel.label, dataChannel.readyState);
        }
    };
    RTCDataChannelDelegateImpl.prototype.dataChannelDidReceiveMessageWithBuffer = function (dataChannel, buffer) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            var type = void 0;
            var data = void 0;
            if (buffer.isBinary) {
                type = webrtc_common_1.WebRTCDataChannelMessageType.BINARY;
                data = buffer.data.base64EncodedStringWithOptions(0);
            }
            else {
                type = webrtc_common_1.WebRTCDataChannelMessageType.TEXT;
                data = NSString.alloc().initWithDataEncoding(buffer.data, NSUTF8StringEncoding);
            }
            owner.delegate.webRTCClientDataChannelMessageType(owner, dataChannel.label, data, type);
        }
    };
    RTCDataChannelDelegateImpl.ObjCProtocols = [RTCDataChannelDelegate];
    return RTCDataChannelDelegateImpl;
}(NSObject));
var RTCPeerConnectionDelegateImpl = (function (_super) {
    __extends(RTCPeerConnectionDelegateImpl, _super);
    function RTCPeerConnectionDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidAddReceiverStreams = function (peerConnection, rtpReceiver, mediaStreams) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidAddStream = function (peerConnection, stream) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.remoteStream = stream;
            dispatch_async(dispatch_get_current_queue(), function () {
                if (stream.videoTracks.count > 0) {
                    owner.remoteTracks.push(stream.videoTracks.firstObject);
                    owner.delegate.webRTCClientDidReceiveRemoteVideoTrackStream(owner, stream.videoTracks.firstObject, stream);
                }
            });
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidChangeIceConnectionState = function (peerConnection, newState) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnIceConnectionChange(owner, newState);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidChangeIceGatheringState = function (peerConnection, newState) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnIceGatheringChange(owner, newState);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidChangeSignalingState = function (peerConnection, stateChanged) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnSignalingChange(owner, stateChanged);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidGenerateIceCandidate = function (peerConnection, candidate) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientDidGenerateIceCandidate(owner, candidate);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidOpenDataChannel = function (peerConnection, dataChannel) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.dataChannels.set(dataChannel.label, dataChannel);
            dataChannel.delegate = RTCDataChannelDelegateImpl.initWithOwner(this._owner);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidRemoveIceCandidates = function (peerConnection, candidates) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnIceCandidatesRemoved(owner, candidates);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidRemoveStream = function (peerConnection, stream) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnRemoveStream(owner, stream);
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionDidStartReceivingOnTransceiver = function (peerConnection, transceiver) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
        }
    };
    RTCPeerConnectionDelegateImpl.prototype.peerConnectionShouldNegotiate = function (peerConnection) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.delegate.webRTCClientOnRenegotiationNeeded(owner);
        }
    };
    RTCPeerConnectionDelegateImpl.initWithOwner = function (owner) {
        var delegate = new RTCPeerConnectionDelegateImpl();
        delegate._owner = owner;
        return delegate;
    };
    RTCPeerConnectionDelegateImpl.ObjCProtocols = [RTCPeerConnectionDelegate];
    return RTCPeerConnectionDelegateImpl;
}(NSObject));
var WebRTCClientDelegate = (function (_super) {
    __extends(WebRTCClientDelegate, _super);
    function WebRTCClientDelegate() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebRTCClientDelegate.initWithOwner = function (owner) {
        var delegate = new WebRTCClientDelegate();
        delegate._owner = owner;
        return delegate;
    };
    WebRTCClientDelegate.prototype.webRTCClientOnRemoveStream = function (client, stream) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnRemoveStream',
                object: observable_1.fromObject({
                    client: owner,
                    stream: stream
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientDataChannelStateChanged = function (client, name, type) {
        var owner = this._owner ? this._owner.get() : null;
        var state;
        switch (type) {
            case RTCDataChannelState.Connecting:
                state = webrtc_common_1.WebRTCDataChannelState.CONNECTING;
                break;
            case RTCDataChannelState.Closed:
                state = webrtc_common_1.WebRTCDataChannelState.CLOSED;
                break;
            case RTCDataChannelState.Closing:
                state = webrtc_common_1.WebRTCDataChannelState.CLOSING;
                break;
            case RTCDataChannelState.Open:
                state = webrtc_common_1.WebRTCDataChannelState.OPEN;
                break;
        }
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientDataChannelStateChanged',
                object: observable_1.fromObject({
                    client: owner,
                    name: name,
                    state: state
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientDataChannelMessageType = function (client, name, data, type) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientDataChannelMessageType',
                object: observable_1.fromObject({
                    client: owner,
                    name: name,
                    message: data,
                    type: type
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientStartCallWithSdp = function (client, sdp) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            var type = void 0;
            switch (sdp.type) {
                case RTCSdpType.Offer:
                    type = webrtc_common_1.WebRTCSdpType.OFFER;
                    break;
                case RTCSdpType.PrAnswer:
                    type = webrtc_common_1.WebRTCSdpType.PRANSWER;
                    break;
                case RTCSdpType.Answer:
                    type = webrtc_common_1.WebRTCSdpType.ANSWER;
                    break;
            }
            owner.notify({
                eventName: 'webRTCClientStartCallWithSdp',
                object: observable_1.fromObject({
                    client: client,
                    sdp: sdp.sdp,
                    type: type
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientDidReceiveRemoteVideoTrackStream = function (client, remoteVideoTrack, stream) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientDidReceiveRemoteVideoTrackStream',
                object: observable_1.fromObject({
                    client: client,
                    remoteVideoTrack: remoteVideoTrack,
                    stream: stream
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientDidReceiveError = function (client, error) {
        var owner = this._owner ? this._owner.get() : null;
        owner.notify({
            eventName: 'webRTCClientDidReceiveRemoteVideoTrackStream',
            object: observable_1.fromObject({
                client: client,
                error: error.localizedDescription
            })
        });
    };
    WebRTCClientDelegate.prototype.webRTCClientOnRenegotiationNeeded = function (client) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnRenegotiationNeeded',
                object: observable_1.fromObject({
                    client: owner
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientDidGenerateIceCandidate = function (client, iceCandidate) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientDidGenerateIceCandidate',
                object: observable_1.fromObject({
                    client: client,
                    iceCandidate: {
                        sdp: iceCandidate.sdp,
                        sdpMid: iceCandidate.sdpMid,
                        sdpMLineIndex: iceCandidate.sdpMLineIndex,
                        serverUrl: iceCandidate.serverUrl
                    }
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnIceCandidatesRemoved = function (client, candidates) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnRenegotiationNeeded',
                object: observable_1.fromObject({
                    client: owner,
                    candidates: candidates
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnIceConnectionChange = function (client, connectionState) {
        var owner = this._owner ? this._owner.get() : null;
        var state;
        switch (connectionState) {
            case RTCIceConnectionState.New:
                state = webrtc_common_1.IceConnectionState.NEW;
                break;
            case RTCIceConnectionState.Checking:
                state = webrtc_common_1.IceConnectionState.CHECKING;
                break;
            case RTCIceConnectionState.Connected:
                state = webrtc_common_1.IceConnectionState.CONNECTED;
                break;
            case RTCIceConnectionState.Completed:
                state = webrtc_common_1.IceConnectionState.COMPLETED;
                break;
            case RTCIceConnectionState.Failed:
                state = webrtc_common_1.IceConnectionState.FAILED;
                break;
            case RTCIceConnectionState.Disconnected:
                state = webrtc_common_1.IceConnectionState.DISCONNECTED;
                break;
            case RTCIceConnectionState.Closed:
                state = webrtc_common_1.IceConnectionState.CLOSED;
                break;
        }
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnIceConnectionChange',
                object: observable_1.fromObject({
                    client: owner,
                    state: state
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnIceConnectionReceivingChange = function (client, change) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnIceConnectionReceivingChange',
                object: observable_1.fromObject({
                    client: owner,
                    change: change
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnIceGatheringChange = function (client, gatheringState) {
        var owner = this._owner ? this._owner.get() : null;
        var state;
        switch (gatheringState) {
            case RTCIceGatheringState.New:
                state = webrtc_common_1.IceGatheringState.NEW;
                break;
            case RTCIceGatheringState.Gathering:
                state = webrtc_common_1.IceGatheringState.GATHERING;
                break;
            case RTCIceGatheringState.Complete:
                state = webrtc_common_1.IceGatheringState.COMPLETE;
                break;
        }
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnIceGatheringChange',
                object: observable_1.fromObject({
                    client: owner,
                    state: state
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnSignalingChange = function (client, signalingState) {
        var owner = this._owner ? this._owner.get() : null;
        var state;
        switch (signalingState) {
            case RTCSignalingState.Closed:
                state = webrtc_common_1.SignalingState.CLOSED;
                break;
            case RTCSignalingState.HaveLocalOffer:
                state = webrtc_common_1.SignalingState.HAVE_LOCAL_OFFER;
                break;
            case RTCSignalingState.HaveLocalPrAnswer:
                state = webrtc_common_1.SignalingState.HAVE_LOCAL_PRANSWER;
                break;
            case RTCSignalingState.HaveRemoteOffer:
                state = webrtc_common_1.SignalingState.HAVE_REMOTE_OFFER;
                break;
            case RTCSignalingState.HaveRemotePrAnswer:
                state = webrtc_common_1.SignalingState.HAVE_REMOTE_PRANSWER;
                break;
            case RTCSignalingState.Stable:
                state = webrtc_common_1.SignalingState.STABLE;
                break;
        }
        owner.notify({
            eventName: 'webRTCClientOnIceGatheringChange',
            object: observable_1.fromObject({
                client: owner,
                state: state
            })
        });
    };
    WebRTCClientDelegate.prototype.webRTCClientOnCameraSwitchDone = function (client, done) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnCameraSwitchDone',
                object: observable_1.fromObject({
                    client: owner,
                    done: done
                })
            });
        }
    };
    WebRTCClientDelegate.prototype.webRTCClientOnCameraSwitchError = function (client, error) {
        var owner = this._owner ? this._owner.get() : null;
        if (owner) {
            owner.notify({
                eventName: 'webRTCClientOnCameraSwitchError',
                object: observable_1.fromObject({
                    client: owner,
                    message: error
                })
            });
        }
    };
    return WebRTCClientDelegate;
}(NSObject));
var TNSRTCScaling;
(function (TNSRTCScaling) {
    TNSRTCScaling["FIT"] = "fit";
    TNSRTCScaling["FILL"] = "fill";
    TNSRTCScaling["NONE"] = "none";
})(TNSRTCScaling = exports.TNSRTCScaling || (exports.TNSRTCScaling = {}));
var WebRTCView = (function (_super) {
    __extends(WebRTCView, _super);
    function WebRTCView() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebRTCView.prototype.createNativeView = function () {
        return FancyWebRTCView.new();
    };
    WebRTCView.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        if (this.nativeView && this._scaling) {
            this.nativeView.setScalingWithScale(this._scaling);
        }
    };
    Object.defineProperty(WebRTCView.prototype, "mirror", {
        set: function (mirror) {
            this.nativeView.setMirrorWithMirror(mirror);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebRTCView.prototype, "videoTrack", {
        set: function (track) {
            this.nativeView.setVideoTrackWithTrack(track);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebRTCView.prototype, "stream", {
        set: function (stream) {
            if (stream instanceof RTCMediaStream) {
                this.nativeView.setSrcObjectWith(stream);
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebRTCView.prototype, "scaling", {
        set: function (scaling) {
            var nativeScaling = 2;
            switch (scaling) {
                case TNSRTCScaling.FILL:
                    nativeScaling = 1;
                    break;
                case TNSRTCScaling.FIT:
                    nativeScaling = 0;
                    break;
                default:
                    break;
            }
            if (this.nativeView) {
                this.nativeView.setScalingWithScale(nativeScaling);
            }
            this._scaling = nativeScaling;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(WebRTCView.prototype, "srcObject", {
        set: function (value) {
            if (!this.nativeView)
                return;
            if (value instanceof ios_1.TNSRTCMediaStream) {
                this.nativeView.setSrcObjectWithStream(value.instance);
            }
            else if (value instanceof RTCMediaStream) {
                this.nativeView.setSrcObjectWith(value);
            }
            else if (value instanceof ios_1.TNSRTCMediaStreamTrack) {
                this.nativeView.setSrcObjectWithFancy(value.instance);
            }
            else if (value instanceof RTCMediaStreamTrack) {
                this.nativeView.setSrcObjectWithRtc(value);
            }
        },
        enumerable: true,
        configurable: true
    });
    WebRTCView.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var nativeView = this.nativeView;
        if (nativeView) {
            var width = view_1.layout.getMeasureSpecSize(widthMeasureSpec);
            var height = view_1.layout.getMeasureSpecSize(heightMeasureSpec);
            this.setMeasuredDimension(width, height);
        }
    };
    return WebRTCView;
}(view_1.View));
exports.WebRTCView = WebRTCView;
var WebRTCCapturer = (function () {
    function WebRTCCapturer(client, capturer, quality) {
        this.cameraPosition = 'front';
        this.client = client;
        this.capturer = capturer;
        this.quality = quality ? quality : webrtc_common_1.Quality.LOWEST;
    }
    WebRTCCapturer.prototype.getCapturer = function () {
        return this.capturer;
    };
    WebRTCCapturer.prototype.start = function () {
        this.startWithError(false);
    };
    WebRTCCapturer.prototype.startWithError = function (show) {
        var _this = this;
        var devices = RTCCameraVideoCapturer.captureDevices();
        var device;
        var pos = this.cameraPosition === 'front'
            ? 2
            : 1;
        for (var i = 0; i < devices.count; i++) {
            if (devices[i].position === pos) {
                device = devices[i];
                break;
            }
        }
        var format = this.selectFormatForDevice(device);
        var fps = this.selectFpsForFormat(format);
        this.capturer.startCaptureWithDeviceFormatFps(device, format, fps);
        this.capturer.startCaptureWithDeviceFormatFpsCompletionHandler(device, format, fps, function (error) {
            var client = _this.client ? _this.client.get() : null;
            if (show && client) {
                if (error) {
                    client.delegate.webRTCClientOnCameraSwitchError(client, error.localizedDescription);
                }
                else {
                    client.delegate.webRTCClientOnCameraSwitchDone(client, true);
                }
            }
        });
    };
    WebRTCCapturer.prototype.selectFpsForFormat = function (format) {
        var maxFrameRate = 0;
        for (var i = 0; i < format.videoSupportedFrameRateRanges.count; i++) {
            var fpsRange = format.videoSupportedFrameRateRanges.objectAtIndex(i);
            maxFrameRate = fmax(maxFrameRate, fpsRange.maxFrameRate);
        }
        return maxFrameRate;
    };
    WebRTCCapturer.prototype.getFormatOrHighestFormat = function (supportedFormats, device) {
        var targetHeight = 144;
        var targetWidth = 192;
        var highest = function () {
            var count = supportedFormats.count;
            var last = CMVideoFormatDescriptionGetDimensions(supportedFormats.firstObject);
            for (var i = 0; i < count; i++) {
                var format = supportedFormats.objectAtIndex(i);
                var dimensions = CMVideoFormatDescriptionGetDimensions(format.description);
                if (last.width < dimensions.width) {
                    last = dimensions;
                }
            }
            return last;
        };
        var best = highest();
        switch (this.quality) {
            case webrtc_common_1.Quality.HIGHEST:
                targetHeight = best.height;
                targetWidth = best.width;
                break;
            case webrtc_common_1.Quality.MAX_480P:
                if (device.supportsAVCaptureSessionPreset(AVCaptureSessionPreset640x480)) {
                    targetHeight = 480;
                    targetWidth = 640;
                }
                else {
                    targetHeight = best.height;
                    targetWidth = best.width;
                }
                break;
            case webrtc_common_1.Quality.MAX_720P:
                if (device.supportsAVCaptureSessionPreset(AVCaptureSessionPreset1280x720)) {
                    targetHeight = 720;
                    targetWidth = 1280;
                }
                else {
                    targetHeight = best.height;
                    targetWidth = best.width;
                }
                break;
            case webrtc_common_1.Quality.MAX_1080P:
                if (device.supportsAVCaptureSessionPreset(AVCaptureSessionPreset1920x1080)) {
                    targetHeight = 1080;
                    targetWidth = 1920;
                }
                else {
                    targetHeight = best.height;
                    targetWidth = best.width;
                }
                break;
            case webrtc_common_1.Quality.MAX_2160P:
                if (device.supportsAVCaptureSessionPreset(AVCaptureSessionPreset3840x2160)) {
                    targetHeight = 2160;
                    targetWidth = 3840;
                }
                else {
                    targetHeight = best.height;
                    targetWidth = best.width;
                }
                break;
            default:
                targetHeight = 144;
                targetWidth = 192;
                break;
        }
        return { width: targetWidth, height: targetHeight };
    };
    WebRTCCapturer.prototype.selectFormatForDevice = function (device) {
        var supportedFormats = RTCCameraVideoCapturer.supportedFormatsForDevice(device);
        var resolution = this.getFormatOrHighestFormat(supportedFormats, device);
        var targetHeight = resolution.height;
        var targetWidth = resolution.width;
        var selectedFormat = null;
        var currentDiff = Number.MAX_VALUE;
        for (var i = 0; i < supportedFormats.count; i++) {
            var format = supportedFormats.objectAtIndex(i);
            var dimension = CMVideoFormatDescriptionGetDimensions(format);
            var diff = abs(targetWidth - dimension.width) +
                abs(targetHeight - dimension.height);
            if (diff < currentDiff) {
                selectedFormat = format;
                currentDiff = diff;
            }
        }
        return selectedFormat;
    };
    WebRTCCapturer.prototype.stop = function () {
        this.capturer.stopCapture();
    };
    WebRTCCapturer.prototype.toggleCamera = function () {
        if (this.cameraPosition === 'front') {
            this.cameraPosition = 'back';
        }
        else {
            this.cameraPosition = 'front';
        }
        this.stop();
        this.startWithError(true);
    };
    return WebRTCCapturer;
}());
//# sourceMappingURL=webrtc.ios.js.map